
#### This script preprocesses Roche 454 Reads
## START Rdev

BaseDir <- "/User_Projects/Wichman/Fluidigm_Oct23_11"

options(width=120)
options("stringsAsFactors" = FALSE)
library(Biobase)
library(rSFFreader)

## slarti
#source("/mnt/home/msettles/darwin_home/projects/Wichman/Fluidigm_Phage/FluidR/R/Stub.R")
#source("/mnt/home/msettles/darwin_home/projects/Wichman/Fluidigm_Phage/FluidR/R/utilityfunctions.R")

source(file.path(BaseDir,"R_scripts/Stub.R"))
source(file.path(BaseDir,"R_scripts/utilityfunctions.R"))


basefilename="CombinedRun"
sffbase <- file.path(BaseDir,"SFF_Files")
sfffiles <- file.path(sffbase,c("G7BP9S302.sff","G4S1EIW02.sff"))

#sfffiles <- dir("SFF_Files",".sff",full.names=TRUE)
primerfile <-  file.path(BaseDir,"config","FluidigmPhagePrimers.txt")
adapterfile <- file.path(BaseDir,"config","Fluidigm454TiMIDS.txt")

primerADF <- read.AnnotatedDataFrame(primerfile)
adapterADF <- read.AnnotatedDataFrame(adapterfile)

sampleMetaData <- "Wichman_fluidigm_9-14-11.txt"

if(file.exists(basefilename)) {
    warning(paste("Directory ",basefilename," already exist and may contain data which will be overwritten",sep=""))
    if(!file.exists(file.path(basefilename,"Figures"))) dir.create(file.path(basefilename,"Figures"))
} else {
    dir.create(basefilename)
    dir.create(file.path(basefilename,"Figures"))
}
figurepath <- file.path(basefilename,"Figures")


outfile <- paste(basefilename,".out",sep="")

# Read in the sff files
sff <- readsff(sfffiles)
clipMode(sff) <- "Raw"
cat(paste("Total Number of Reads in SFF files:",length(sff),"\n"),file=outfile,append=F)    
#####################
### Base Roche stats and clip
print("Identifying Roche Clip points")
ReadData <- data.frame(Acc=as.character(id(sff)),
						Run=basefilename,
						Run_454_ID=as.character(subseq(id(sff),1,7)),
						Region=as.character(subseq(id(sff),8,9)),
						Loc=as.character(subseq(id(sff),10,14)),
						RawLength=width(sff))

ReadData$RocheLC <- start(qualityClip(sff))
ReadData$RocheRC <- end(qualityClip(sff))

ReadData$RocheLength <- width(sread(sff,clipmode="Full"))

cat(paste("Mean length before Roche Right Clip",signif(mean(ReadData$RawLength),3),"\n"),file=outfile,append=T)    
cat(paste("Median length before Roche Right Clip",signif(median(ReadData$RawLength),3),"\n"),file=outfile,append=T)    
cat(paste("Mean length after Roche Right Clip",signif(mean(ReadData$RocheLength),3),"\n"),file=outfile,append=T)    
cat(paste("Median length after Roche Right Clip",signif(median(ReadData$RocheLength),3),"\n"),file=outfile,append=T)    

### End Base Roche
#####################
###
#####################
### Run cross_match looking for primers and tags
###replace the initial tcag with ---- to prevent matching
#system(paste("cp ",basefilename,".fasta ",basefilename,"_orig.fasta",sep=""))
#system(paste("sed -i 's/^tcag/----/g' ",basefilename,".fasta",sep=""))

#print("Running cross_match, looking for adapter and primers")
#system(paste("cross_match TMP.raw.fasta ../",screenfile," -minmatch 15 -minscore 14 -tags > TMP.cross_match.out",sep=""))
#system("grep \"ALIGNMENT\" TMP.cross_match.out | replace ' C ' ' C_' --  > TMP.cm_alignment.out")
# copy back over the original file
#system(paste("mv ",basefilename,"_orig.fasta ",basefilename,".fasta",sep=""))
#cm_out <- parse_cm("TMP.cross_match.out")
writeFastaQual(sff,basefilename="TMP.raw")
cm_primer <- cross_match_search("TMP.raw.fasta",primerADF,"primer")
cm_adapt <- cross_match_search("TMP.raw.fasta",adapterADF,"adapter")

# ssaha2 -output sam -outfile ssaha2.sam -kmer 8 -skip 1 -seeds 1 -score 28 -cmatch 9 -ckmer 6 screenfile.fasta TMP.raw.fastq 
### End CrossMatch
############################
###
############################
### Adapter Clip Points
ReadData$AdapterLC <- ReadData$RocheLC
ReadData$AdapterRC <- ReadData$RocheRC

print("Parsing cross_match output")
### SET new clip points, recording primer information

#### New Left Clip point starts after the inner most Forward Primer
tmp <-tapply(cm_primer$read_end[(cm_primer$FC == "F")],cm_primer$read_id[(cm_primer$FC == "F")],max)
ReadData$AdapterLC[match(names(tmp),ReadData$Acc)] <-
    pmax(ReadData$AdapterLC[match(names(tmp),ReadData$Acc)],tmp) +1 ### do I still need the +1

#### New Right Clip points occurs before the inner most Reverse Primers
tmp <-tapply(cm_primer$read_start[(cm_primer$FC == "C")],cm_primer$read_id[(cm_primer$FC == "C")],min)
ReadData$AdapterRC[match(names(tmp),ReadData$Acc)] <-
    pmin(ReadData$AdapterRC[match(names(tmp),ReadData$Acc)],tmp) -1

ReadData$AdapterLC <- pmin(ReadData$AdapterLC,ReadData$RawLength)
ReadData$AdapterRC <- pmax(ReadData$AdapterLC,ReadData$AdapterRC)
ReadData$AdapterLength <- ReadData$AdapterRC - ReadData$AdapterLC +1

## idendify reads with duplicate matches to Forward or Reverse Primers
readsWdups <- unique(c( cm_primer$read_id[which(cm_primer$FC == "F")][duplicated(cm_primer$read_id[which(cm_primer$FC == "F")])], ## forward
                        cm_primer$read_id[which(cm_primer$FC == "C")][duplicated(cm_primer$read_id[which(cm_primer$FC == "C")])],
                        cm_adapt$read_id[which(cm_adapt$FC == "F")][duplicated(cm_adapt$read_id[which(cm_adapt$FC == "F")])], ## forward
                        cm_adapt$read_id[which(cm_adapt$FC == "C")][duplicated(cm_adapt$read_id[which(cm_adapt$FC == "C")])])) ## reverse

cat(paste("Identified ::", length(readsWdups),":: reads with multiple adapters or primers, these are likely primer dimer sequences and are removed\n"),file=outfile,append=T)

## remove reads that have multiple matches to forward or reverse probes
if (length(readsWdups) > 0){
    cm_primer <- cm_primer[-which(cm_primer$read_id %in% readsWdups),]
    cm_adapt <- cm_adapt[-which(cm_adapt$read_id %in% readsWdups),]
    ReadData$AdapterRC[match(readsWdups,ReadData$Acc)] <- ReadData$AdapterLC[match(readsWdups,ReadData$Acc)] ## effectively Remove
}

## Calculate Primer errors
print("Computing, primer errors")
ReadData$FPrimer_Code <- NA
ReadData$FPrimer_Code[match(cm_primer$read_id[which(cm_primer$FC == "F")],ReadData$Acc)] <- cm_primer$adapt[which(cm_primer$FC == "F")]
ReadData$FPErr <- 100
ReadData$FPErr[match(cm_primer$read_id[which(cm_primer$FC == "F")],ReadData$Acc)] <- cm_primer[which(cm_primer$FC == "F"),"err"]

cat("Forward Primer Errors:\n",file=outfile,append=T)
sink(file=outfile,append=TRUE)
stem(ReadData$FPErr,scale=0.5,width=10)
sink()
cat("\n",file=outfile,append=T)

ReadData$RPrimer_Code <- NA
ReadData$RPrimer_Code[match(cm_primer$read_id[which(cm_primer$FC == "C")],ReadData$Acc)] <- cm_primer$adapt[which(cm_primer$FC == "C")]
ReadData$RPErr <- 100
ReadData$RPErr[match(cm_primer$read_id[which(cm_primer$FC == "C")],ReadData$Acc)] <- cm_primer$err[which(cm_primer$FC == "C")]

cat("Reverse Primer Errors:\n",file=outfile,append=T)
sink(file=outfile,append=TRUE)
stem(ReadData$RPErr,scale=0.5,width=10)
sink()
cat("\n",file=outfile,append=T)


print("Computing, tags and tagged adapter errors")
ReadData$FTag_Code <- NA
ReadData$FTag_Code[match(cm_adapt$read_id[which(cm_adapt$FC == "F")],ReadData$Acc)] <- cm_adapt$adapt[which(cm_adapt$FC == "F")]
ReadData$FTErr <- 100
ReadData$FTErr[match(cm_adapt$read_id[which(cm_adapt$FC == "F")],ReadData$Acc)] <- cm_adapt[which(cm_adapt$FC == "F"),"err"]

cat("Forward Tag Errors:\n",file=outfile,append=T)
sink(file=outfile,append=TRUE)
stem(ReadData$FTErr,scale=0.5,width=10)
sink()
cat("\n",file=outfile,append=T)

print("Computing, tags and tagged primer errors")
ReadData$RTag_Code <- NA
ReadData$RTag_Code[match(cm_adapt$read_id[which(cm_adapt$FC == "C")],ReadData$Acc)] <- cm_adapt$adapt[which(cm_adapt$FC == "C")]
ReadData$RTErr <- 100
ReadData$RTErr[match(cm_adapt$read_id[which(cm_adapt$FC == "C")],ReadData$Acc)] <- cm_adapt[which(cm_adapt$FC == "C"),"err"]

cat("Reverse Tag Errors:\n",file=outfile,append=T)
sink(file=outfile,append=TRUE)
stem(ReadData$RTErr,scale=0.5,width=10)
sink()
cat("\n",file=outfile,append=T)

### fixing end points
## any code == NA set the RC point to the LC point
ReadData$AdapterRC[is.na(ReadData$FPrimer_Code)] <- ReadData$AdapterLC[is.na(ReadData$FPrimer_Code)]
ReadData$AdapterRC[is.na(ReadData$RPrimer_Code)] <- ReadData$AdapterLC[is.na(ReadData$RPrimer_Code)]
ReadData$AdapterRC[is.na(ReadData$FTag_Code)] <- ReadData$AdapterLC[is.na(ReadData$FTag_Code)]
ReadData$AdapterRC[is.na(ReadData$RTag_Code)] <- ReadData$AdapterLC[is.na(ReadData$RTag_Code)]
## right clip point cannot be less than left clip point
ReadData$AdapterRC <- pmin(ReadData$AdapterRC,ReadData$RawLength)
ReadData$AdapterRC <- pmax(ReadData$AdapterRC,ReadData$AdapterLC)
ReadData$AdapterLength <- ReadData$AdapterRC - ReadData$AdapterLC +1


FPcode <- primerADF$TargetName[match(ReadData$FPrimer_Code,featureNames(primerADF))]
RPcode <- primerADF$TargetName[match(ReadData$RPrimer_Code,featureNames(primerADF))]

Fwd <- primerADF$Orientation[ReadData$FPrimer_Code,featureNames(primerADF))];
Rvs <- primerADF$Orientation[ReadData$RPrimer_Code,featureNames(primerADF))]

Pdir <- apply(cbind(Fwd,Rvs),1, 
    function(x) {
        switch(EXPR=paste(x,collapse="",sep=""),FR="F",RF="C",NA);
})


FTcode <- adapterADF$MIDtag[match(ReadData$FTag_Code,featureNames(adapterADF))]
RTcode <- adapterADF$MIDtag[match(ReadData$RTag_Code,featureNames(adapterADF))]

Fwd <- adapterADF$Orientation[match(ReadData$FTag_Code,featureNames(adapterADF))];
Rvs <- adapterADF$Orientation[match(ReadData$RTag_Code,featureNames(adapterADF))];

Tdir <- apply(cbind(Fwd,Rvs),1, 
    function(x) {
        switch(EXPR=paste(x,collapse="",sep=""),FR="F",RF="C",NA);
})

sink(file=outfile,append=TRUE)
table(Pdir,Tdir,useNA="always")
sink()

sampleID <- apply(cbind(ReadData$Run_454_ID,as.numeric(ReadData$Region),FTcode,RTcode,Tdir),1, 
    function(x) 
        switch(x[5],F=paste(x[1],x[2],x[3],x[4],sep=" "),C=paste(x[1],x[2],x[4],x[3],sep=" "),NA)) 
names(sampleID) <- NULL

samples <- read.table(sampleMetaData,sep="\t",header=T,as.is=T)
samples$BC_ID <- apply(samples[,c("X454_Run","X454_Location","BC_F","BC_R")],1,paste,collapse=" ")

ReadData$Sample <- samples$Sample_ID[match (sampleID,samples$BC_ID)]
ReadData$Amplicon <- ifelse(FPcode == RPcode,FPcode, NA) 
ReadData$Dir <- ifelse(Tdir == Pdir,Tdir, NA)


ReadData$GoodRead <- FPcode == RPcode & 
                     !is.na(FPcode) & 
                     !is.na(RPcode) & 
                     (Tdir == Pdir) & 
                     !is.na(ReadData$Sample) & 
                     (ReadData$FPErr < 3 & ReadData$FTErr < 3 +21 & ReadData$RPErr < 5 & ReadData$RTErr < 5) &
                     (ReadData$AdapterLength > 100)

ReadData$GoodRead[is.na(ReadData$GoodRead)] <- FALSE

### Write out data
primerRange <- IRanges(start=ReadData$AdapterLC,ReadData$AdapterRC)
#adapterClip(sff) <- primerRange ## need to add replacement function 
sff@adapterClip <- primerRange

newsff <- reverseComplement(sff,ReadData$Dir == "C" & !is.na(ReadData$Dir))
newsff <- newsff[order(ReadData$Sample,ReadData$Amplicon)]
ReadData <- ReadData[order(ReadData$Sample,ReadData$Amplicon),]

write.table(ReadData,file="ReadData.txt",sep="\t",row.names=F,col.names=T,quote=F)

writeFastaQual(newsff[ReadData$GoodRead],file="good_reads.fasta")
writeFastq(newsff[ReadData$GoodRead],file="good_reads.fastq")
write.table(ReadData[ReadData$GoodRead,],file="good_data.txt",sep="\t",row.names=F,col.names=T,quote=F)

sapply(unique(samples$Investigator),dir.create)

clipMode(newsff) <- "Full"
for (i in na.exclude(unique(ReadData$Sample))){
	folder <- samples[which(samples$Sample_ID == i),"Investigator"][1]
	writeFastq(newsff[ReadData$GoodRead & ReadData$Sample == i],file=file.path(folder,paste(i,".fastq",sep="")))

	system(paste("gmap -d AncGenomes -f samse ", folder,"/",i,".fastq > ",folder,"/",i,".sam",sep=""))
	system(paste("samtools view -b -S ", folder,"/",i,".sam > ", folder,"/",i,".bam",sep=""))
	system(paste("samtools sort ", folder,"/",i,".bam ", folder,"/",i,".sort",sep=""))
	system(paste("samtools mpileup -f config/AncGenomes.fa ", folder,"/",i,".sort.bam > ", folder,"/",i,".sort.pileup",sep=""))

#	system(paste("samtools mpileup -g -f config/AncGenomes.fa ", folder,"/",i,".sort.bam | bcftools view - > ", folder,"/",i,".vcf",sep=""))
	system(paste("samtools mpileup -g -f config/AncGenomes.fa ", folder,"/",i,".sort.bam | bcftools view -c -v -g - > ", folder,"/",i,".vcf",sep=""))
	
}

vcfheader <- c("CHROM","POS","ID","REF","ALT","QUAL","FILTER","INFO","FORMAT","NOTE")
pileupheader <- c("CHROM","POS","REF","COVERAGE","BASE","QUAL")

## WichmanSamples
library(stringr)

folder = "Wichman"

####### ID11 Isolates
wichman <- samples[which(samples$Investigator=="Wichman" & samples$population==FALSE & samples$Project_ID == "ID11"),]
wichman <- wichman[wichman$Sample_ID %in% unique(ReadData$Sample),]

## remove those not real
exclude <- names(table(ReadData[which(ReadData$Sample %in% wichman$Sample_ID),"Sample"]))[table(ReadData[which(ReadData$Sample %in% wichman$Sample_ID),"Sample"]) < 5]
if (length(exclude)) wichman <- wichman[!(wichman$Sample %in% exclude),]

CoverageMap <- matrix(0,ncol=nrow(wichman),nrow=5577)
colnames(CoverageMap) <- wichman$Sample_ID
vcfInfo <- data.frame(stringsAsFactors=F)
pdf("ID11GenomeCoverage.pdf",width=6,height=3)
for (i in wichman$Sample_ID){
        null <- numeric(5577)
	pileup <- read.table(paste(folder,"/",i,".sort.pileup",sep=""),quote="",comment.char="",as.is=T)
	colnames(pileup) <- pileupheader
	pileup <- pileup[pileup$CHROM == "ID11Anc",]
	null[as.numeric(pileup$POS)] <- as.numeric(pileup$COVERAGE)
	plot(seq.int(1,5577),null,type="l",main=i,xlab="base pair position",ylab="coverage",ylim=c(0,max(null)+2))
	CoverageMap[,i] <- null

	vcf <- try(read.table(paste(folder,"/",i,".vcf",sep=""),quote="",comment.char="#",as.is=T,
                   colClasses=c("character","integer",rep("character",times=3),"numeric",rep("character",times=4))))
	if (!(class(vcf) == "try-error")){
		colnames(vcf) <- vcfheader
		vcf <- vcf[vcf$CHROM == "ID11Anc",]
		vcf$SAMPLE <- i
        	vcf$INDEL <- FALSE
        	vcf$INDEL[grep ("INDEL",vcf$INFO)] <- TRUE
		vcf$COVERAGE = as.numeric(sub("DP=","",str_extract(vcf$INFO,"DP=[[:digit:]]+")))
        	vcf$GENOTYPE = substring(vcf$NOTE,1,3)
		vcfInfo <- rbind(vcfInfo,vcf[,c(11,1,2,4,5,6,12,13,14,8,9,10)])
	}
}
dev.off()
colnames(vcfInfo) <- c("SAMPLE","CHROM","POS","REF","ALT","QUAL","INDEL","COVERAGE","GENOTYPE","INFO","FORMAT","NOTE")

vcfGood <- vcfInfo[(vcfInfo$GENOTYPE == "1/1" & vcfInfo$COVERAGE > 4),]

ImageMap <- matrix(NA,ncol=nrow(wichman),nrow=5577)
colnames(ImageMap) <- wichman$Sample_ID
for (i in seq.int(1,nrow(vcfGood))){ ImageMap[as.numeric(vcfGood[i,3]),vcfGood[i,1]] <- switch(vcfGood[i,5],A=1,C=2,T=3,G=4,5)}
pdf("ID11VariantsImage.pdf")
image(ImageMap,col=c("green","blue","red","black","yellow"))
dev.off()

write.table(vcfGood,file="ID11variantGood.txt",sep="\t",col.names=T,row.names=F,quote=F)    
write.table(vcfInfo,file="ID11variantAll.txt",sep="\t",col.names=T,row.names=F,quote=F)    
write.table(CoverageMap,file="ID11CoverageMap.txt",sep="\t",col.names=T,row.names=F,quote=F)    
write.table(wichman,file="ID11expParams.txt",sep="\t",col.names=T,row.names=F,quote=F)    

#### PhiX Isolates
wichman <- samples[which(samples$Investigator=="Wichman" & samples$population==FALSE & samples$Primer_Set == "PhiXab"),]
wichman <- wichman[wichman$Sample_ID %in% unique(ReadData$Sample),]

## remove those not real
exclude <- names(table(ReadData[which(ReadData$Sample %in% wichman$Sample_ID),"Sample"]))[table(ReadData[which(ReadData$Sample %in% wichman$Sample_ID),"Sample"]) < 5]
if (length(exclude)) wichman <- wichman[!(wichman$Sample %in% exclude),]

CoverageMap <- matrix(0,ncol=nrow(wichman),nrow=5386)
colnames(CoverageMap) <- wichman$Sample_ID
vcfInfo <- data.frame(stringsAsFactors=F)
pdf("PhiXGenomeCoverage.pdf",width=6,height=3)
for (i in wichman$Sample_ID){
        null <- numeric(5386)
	pileup <- read.table(paste(folder,"/",i,".sort.pileup",sep=""),quote="",comment.char="",as.is=T)
	colnames(pileup) <- pileupheader
	pileup <- pileup[pileup$CHROM == "phiXAnc",]
	null[as.numeric(pileup$POS)] <- as.numeric(pileup$COVERAGE)
	plot(seq.int(1,5386),null,type="l",main=i,xlab="base pair position",ylab="coverage",ylim=c(0,max(null)+2))
	CoverageMap[,i] <- null


	vcf <- try(read.table(paste(folder,"/",i,".vcf",sep=""),quote="",comment.char="#",as.is=T,
                   colClasses=c("character","integer",rep("character",times=3),"numeric",rep("character",times=4))))
	if (!(class(vcf) == "try-error")){
		colnames(vcf) <- vcfheader
		vcf <- vcf[vcf$CHROM == "phiXAnc",]
		vcf$SAMPLE <- i
        	vcf$INDEL <- FALSE
        	vcf$INDEL[grep ("INDEL",vcf$INFO)] <- TRUE
		vcf$COVERAGE = as.numeric(sub("DP=","",str_extract(vcf$INFO,"DP=[[:digit:]]+")))
        	vcf$GENOTYPE = substring(vcf$NOTE,1,3)
		vcfInfo <- rbind(vcfInfo,vcf[,c(11,1,2,4,5,6,12,13,14,8,9,10)])
	}
}
dev.off()
colnames(vcfInfo) <- c("SAMPLE","CHROM","POS","REF","ALT","QUAL","INDEL","COVERAGE","GENOTYPE","INFO","FORMAT","NOTE")

vcfGood <- vcfInfo[(vcfInfo$GENOTYPE == "1/1" & vcfInfo$COVERAGE > 4),]

ImageMap <- matrix(NA,ncol=nrow(wichman),nrow=5386)
colnames(ImageMap) <- wichman$Sample_ID
for (i in seq.int(1,nrow(vcfGood))){ ImageMap[as.numeric(vcfGood[i,3]),vcfGood[i,1]] <- switch(vcfGood[i,5],A=1,C=2,T=3,G=4,5)}
pdf("PhiXVariantsImage.pdf")
image(ImageMap,col=c("green","blue","red","black","yellow"))
dev.off()

vcfNotes <- readLines("vcfNotes.txt",comment.char="")
write.table(vcfGood,file="PhiXvariantGood.txt",sep="\t",col.names=T,row.names=F,quote=F)    
write.table(vcfInfo,file="PhiXvariantAll.txt",sep="\t",col.names=T,row.names=F,quote=F)    
write.table(CoverageMap,file="PhiXCoverageMap.txt",sep="\t",col.names=T,row.names=F,quote=F)    
write.table(wichman,file="PhiXexpParams.txt",sep="\t",col.names=T,row.names=F,quote=F)    


write.table(ReadData,file="ReadData.txt",sep="\t",row.names=F,col.names=T,quote=F)

########################################

primerCounts <- table(paste(ReadData$Sample[ReadData$GoodRead],sampleID[ReadData$GoodRead]),ReadData$Amplicon[ReadData$GoodRead])
primerCounts <- primerCounts[,order(substring(colnames(primerCounts),1,1),as.numeric(sub('p|a|b','',colnames(primerCounts))))]
primerCounts <- primerCounts[match(paste(samples$Sample_ID,samples$BC_ID),rownames(primerCounts)),]
rownames(primerCounts) <- paste(samples$Sample_ID,samples$BC_ID)

primers <- c(paste("a",1:24,sep=""),paste("b",1:24,sep=""),paste("p",1:24,sep=""))
primerCounts <- primerCounts[,match(primers,colnames(primerCounts))]
colnames(primerCounts) <- primers

library(gplots)
pdf("AmpliconCoverage.pdf",pointsize=8)
null <- tapply(1:nrow(samples),samples$Fluidigm_BC,function(x) {
    chip <- samples$Fluidigm_BC[x[1]]
    heatmap.2(primerCounts[x,], Rowv=FALSE, Colv=FALSE,dendrogram="none",
             scale="none", na.rm = TRUE,
             ColSideColors=c(rep("red",24),rep("blue",24),rep("green",24)),
             margins=c(5,15),
             main = chip, xlab = "Primers", ylab = "Samples",col=rainbow(50,start=0,end=0.9),trace="none",density.info="none",keysize=0.75)
})
# hist(unlist(table(ReadData$Sample[ReadData$GoodRead],ReadData$Amplicon[ReadData$GoodRead])),breaks=200,main="Amplicon Coverages",ylab="Number of Amplicons",xlab="Coverage")
dev.off()


write.table(data.frame(samples,primerCounts),file="AmpliconSampleCounts.txt",sep="\t",row.names=FALSE,col.names=TRUE,quote=F)

ReadData$FluidigmPlate <- samples$Fluidigm_BC[match(ReadData$Sample,samples$Sample_ID)

########################################

########################################
########################################
########################################
########################################
########################################
########################################
## Compute consensus sequences 
consSeqs <- tapply(seq.int(1,nrow(ReadData2)),ReadData2$Sample,function(x) tapply(x,ReadData2$Amplicon[x],function(x) {
    sequences <- seq_sub2[x]
    size<-names(sort(table(width(sequences)),decreasing=T))[1]
    newseqs <- sequences[width(sequences)==size]
    c(consensusString(newseqs),length(newseqs),length(sequences))
}))

tb <- cbind(rep(names(consSeqs),times=sapply(consSeqs,length)),unlist(sapply(consSeqs,names)),unlist(sapply(consSeqs,function(x) sapply(x,function(y) y[2]))),unlist(sapply(consSeqs,function(x) sapply(x,function(y) y[3]))),unlist(sapply(consSeqs,function(x) sapply(x,function(y) y[1]))))
tb <- tb[order(tb[,1],as.numeric(sub('p|a|b','',tb[,2]))),]
colnames(tb) <- c("Sample","Primer","Number of Reads Contributing to Consensus","Total Number of Reads","Sequence")

write.table(tb,file="consensus_sequences.txt",sep="\t",row.names=F,col.names=T,quote=F)

final_seqs <- DNAStringSet(tb[,5])
names(final_seqs) <- apply(tb[,c("Sample","Primer")],1,paste,collapse="-")
write.XStringSet(final_seqs,file="consensus_sequences.fasta")


### STEVE's DATA

steve <- which(substring(ReadData2$Sample,1,3) == "40W")

write.XStringSet(seq_sub2[steve],file="steve_good_reads.fasta")
writeFastq(new_fq2[steve],file="steve_good_reads.fastq")
write.table(ReadData2[steve,],file="steve_good_data.txt",sep="\t",row.names=F,col.names=T,quote=F)

splitdata <- split(new_fq2[steve],ReadData2$Sample[steve])
for (i in names(splitdata)) writeFastq(splitdata[[i]],file=paste(i,"fastq",sep="."))
################################# ####################################################################3
### END HERE

